@@ -19,6 +19,7 @@
    is generally kept in a file called COPYING or LICENSE.  If you do not
    have a copy of the license, write to the Free Software Foundation,
    675 Mass Ave, Cambridge, MA 02139, USA. */
+#define READLINE_LIBRARY
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -57,19 +58,24 @@ extern int errno;
 extern char *strchr (), *strrchr ();
 #endif /* !strchr && !__STDC__ */
 
-extern char *tilde_expand ();
-
 extern int _rl_horizontal_scroll_mode;
 extern int _rl_mark_modified_lines;
-extern int _rl_prefer_visible_bell;
+extern int _rl_bell_preference;
 extern int _rl_meta_flag;
-extern int rl_blink_matching_paren;
 extern int _rl_convert_meta_chars_to_ascii;
+extern int _rl_output_meta_chars;
+extern int _rl_complete_show_all;
+#if defined (PAREN_MATCHING)
+extern int rl_blink_matching_paren;
+#endif /* PAREN_MATCHING */
 #if defined (VISIBLE_STATS)
 extern int rl_visible_stats;
 #endif /* VISIBLE_STATS */
 extern int rl_complete_with_tilde_expansion;
 extern int rl_completion_query_items;
+#if defined (VI_MODE)
+extern char *rl_vi_comment_begin;
+#endif
 
 extern int rl_explicit_arg;
 extern int rl_editing_mode;
@@ -80,8 +86,17 @@ extern char *possible_control_prefixes[], *possible_meta_prefixes[];
 
 extern char **rl_funmap_names ();
 
+/* Forward declarations */
+void rl_set_keymap_from_edit_mode ();
+
 static int glean_key_from_name ();
+
+#if defined (HAVE_STRCASECMP)
+#define stricmp strcasecmp
+#define strnicmp strncasecmp
+#else
 static int stricmp (), strnicmp ();
+#endif
 
 #if defined (STATIC_MALLOC)
 static char *xmalloc (), *xrealloc ();
@@ -106,6 +121,7 @@ rl_add_defun (name, function, key)
   if (key != -1)
     rl_bind_key (key, function);
   rl_add_funmap_entry (name, function);
+  return 0;
 }
 
 /* Bind KEY to FUNCTION.  Returns non-zero if KEY is out of range. */
@@ -121,12 +137,9 @@ rl_bind_key (key, function)
     {
       if (_rl_keymap[ESC].type == ISKMAP)
 	{
-#if defined (CRAY)
-	  Keymap escmap = (Keymap)((int)_rl_keymap[ESC].function);
-#else
-	  Keymap escmap = (Keymap)_rl_keymap[ESC].function;
-#endif
+	  Keymap escmap;
 
+	  escmap = FUNCTION_TO_KEYMAP (_rl_keymap, ESC);
 	  key = UNMETA (key);
 	  escmap[key].type = ISFUNC;
 	  escmap[key].function = function;
@@ -184,7 +197,7 @@ rl_set_key (keyseq, function, map)
      Function *function;
      Keymap map;
 {
-  rl_generic_bind (ISFUNC, keyseq, function, map);
+  return (rl_generic_bind (ISFUNC, keyseq, function, map));
 }
 
 /* Bind the key sequence represented by the string KEYSEQ to
@@ -250,7 +263,7 @@ rl_generic_bind (type, keyseq, data, map)
 	{
 	  ic = UNMETA (ic);
 	  if (map[ESC].type == ISKMAP)
-	    map = (Keymap) map[ESC].function;
+	    map = FUNCTION_TO_KEYMAP (map, ESC);
 	}
 
       if ((i + 1) < keys_len)
@@ -261,16 +274,16 @@ rl_generic_bind (type, keyseq, data, map)
 		free ((char *)map[ic].function);
 
 	      map[ic].type = ISKMAP;
-	      map[ic].function = (Function *)rl_make_bare_keymap ();
+	      map[ic].function = KEYMAP_TO_FUNCTION (rl_make_bare_keymap());
 	    }
-	  map = (Keymap)map[ic].function;
+	  map = FUNCTION_TO_KEYMAP (map, ic);
 	}
       else
 	{
 	  if (map[ic].type == ISMACR)
 	    free ((char *)map[ic].function);
 
-	  map[ic].function = (Function *)data;
+	  map[ic].function = KEYMAP_TO_FUNCTION (data);
 	  map[ic].type = type;
 	}
     }
@@ -389,7 +402,7 @@ rl_function_of_keyseq (keyseq, map, type)
 	    }
 	  else
 	    {
-	      map = (Keymap)map[ESC].function;
+	      map = FUNCTION_TO_KEYMAP (map, ESC);
 	      ic = UNMETA (ic);
 	    }
 	}
@@ -406,7 +419,7 @@ rl_function_of_keyseq (keyseq, map, type)
 	      return (map[ic].function);
 	    }
 	  else
-	    map = (Keymap)map[ic].function;
+	    map = FUNCTION_TO_KEYMAP (map, ic);
 	}
       else
 	{
@@ -416,6 +429,7 @@ rl_function_of_keyseq (keyseq, map, type)
 	  return (map[ic].function);
 	}
     }
+  return ((Function *) NULL);
 }
 
 /* The last key bindings file read. */
@@ -425,15 +439,19 @@ static char *last_readline_init_file = (char *)NULL;
 rl_re_read_init_file (count, ignore)
      int count, ignore;
 {
-  return (rl_read_init_file ((char *)NULL));
+  int r;
+  r = rl_read_init_file ((char *)NULL);
+  rl_set_keymap_from_edit_mode ();
+  return r;
 }
 
-/* The final, last-ditch effort file name for an init file. */
-#define DEFAULT_INPUTRC "~/.inputrc"
-
 /* Do key bindings from a file.  If FILENAME is NULL it defaults
-   to `~/.inputrc'.  If the file existed and could be opened and
-   read, 0 is returned, otherwise errno is returned. */
+   to the first non-null filename from this list:
+     1. the filename used for the previous call
+     2. the value of the shell variable `INPUTRC'
+     3. ~/.inputrc
+   If the file existed and could be opened and read, 0 is returned,
+   otherwise errno is returned. */
 int
 rl_read_init_file (filename)
      char *filename;
@@ -446,12 +464,16 @@ rl_read_init_file (filename)
   /* Default the filename. */
   if (!filename)
     {
-      if (last_readline_init_file)
-	filename = last_readline_init_file;
-      else
+      filename = last_readline_init_file;
+      if (!filename)
+        filename = getenv ("INPUTRC");
+      if (!filename)
 	filename = DEFAULT_INPUTRC;
     }
 
+  if (!*filename)
+    filename = DEFAULT_INPUTRC;
+
   openname = tilde_expand (filename);
 
   if ((stat (openname, &finfo) < 0) ||
@@ -463,10 +485,13 @@ rl_read_init_file (filename)
   else
     free (openname);
 
-  if (last_readline_init_file)
-    free (last_readline_init_file);
+  if (filename != last_readline_init_file)
+    {
+      if (last_readline_init_file)
+	free (last_readline_init_file);
 
-  last_readline_init_file = savestring (filename);
+      last_readline_init_file = savestring (filename);
+    }
 
   /* Read the file into BUFFER. */
   buffer = (char *)xmalloc ((int)finfo.st_size + 1);
@@ -488,6 +513,13 @@ rl_read_init_file (filename)
       /* Mark end of line. */
       line[i] = '\0';
 
+      /* Skip leading whitespace. */
+      while (*line && whitespace (*line))
+        {
+	  line++;
+	  i--;
+        }
+
       /* If the line is not a comment, then parse it. */
       if (*line && *line != '#')
 	rl_parse_and_bind (line);
@@ -553,7 +585,7 @@ parser_if (args)
 
       /* Terminals like "aaa-60" are equivalent to "aaa". */
       tname = savestring (rl_terminal_name);
-      tem = strrchr (tname, '-');
+      tem = strchr (tname, '-');
       if (tem)
 	*tem = '\0';
 
@@ -683,9 +715,6 @@ handle_parser_directive (statement)
   return (1);
 }
 
-/* Ugly but working hack for binding prefix meta. */
-#define PREFIX_META_HACK
-
 static int substring_member_of_array ();
 
 /* Read the binding command from STRING and perform it.
@@ -929,10 +958,13 @@ static struct {
 } boolean_varlist [] = {
   { "horizontal-scroll-mode",	&_rl_horizontal_scroll_mode },
   { "mark-modified-lines",	&_rl_mark_modified_lines },
-  { "prefer-visible-bell",	&_rl_prefer_visible_bell },
   { "meta-flag",		&_rl_meta_flag },
+#if defined (PAREN_MATCHING)
   { "blink-matching-paren",	&rl_blink_matching_paren },
+#endif
   { "convert-meta",		&_rl_convert_meta_chars_to_ascii },
+  { "show-all-if-ambiguous",	&_rl_complete_show_all },
+  { "output-meta",		&_rl_output_meta_chars },
 #if defined (VISIBLE_STATS)
   { "visible-stats",		&rl_visible_stats },
 #endif /* VISIBLE_STATS */
@@ -971,10 +1003,6 @@ rl_variable_bind (name, value)
 #if defined (VI_MODE)
 	  _rl_keymap = vi_insertion_keymap;
 	  rl_editing_mode = vi_mode;
-#else
-#if defined (NOTDEF)
-	  ding ();
-#endif /* NOTDEF */
 #endif /* VI_MODE */
 	}
       else if (strnicmp (value, "emacs", 5) == 0)
@@ -988,8 +1016,6 @@ rl_variable_bind (name, value)
   else if (stricmp (name, "comment-begin") == 0)
     {
 #if defined (VI_MODE)
-      extern char *rl_vi_comment_begin;
-
       if (*value)
 	{
 	  if (rl_vi_comment_begin)
@@ -1010,6 +1036,37 @@ rl_variable_bind (name, value)
 	}
       rl_completion_query_items = nval;
     }
+  else if (stricmp (name, "keymap") == 0)
+    {
+      Keymap kmap;
+      kmap = rl_get_keymap_by_name (value);
+      if (kmap)
+        rl_set_keymap (kmap);
+    }
+  else if (stricmp (name, "bell-style") == 0)
+    {
+      if (!*value)
+        _rl_bell_preference = AUDIBLE_BELL;
+      else
+        {
+          if (stricmp (value, "none") == 0 || stricmp (value, "off") == 0)
+            _rl_bell_preference = NO_BELL;
+          else if (stricmp (value, "audible") == 0 || stricmp (value, "on") == 0)
+            _rl_bell_preference = AUDIBLE_BELL;
+          else if (stricmp (value, "visible") == 0)
+            _rl_bell_preference = VISIBLE_BELL;
+        }
+    }
+  else if (stricmp (name, "prefer-visible-bell") == 0)
+    {
+      /* Backwards compatibility. */
+      if (*value && (stricmp (value, "on") == 0 ||
+		     (*value == '1' && !value[1])))
+        _rl_bell_preference = VISIBLE_BELL;
+      else
+        _rl_bell_preference = AUDIBLE_BELL;
+    }
+
   return 0;
 }
 
@@ -1117,8 +1174,8 @@ rl_set_keymap_from_edit_mode ()
 
 /* Print the names of functions known to Readline. */
 void
-rl_list_funmap_names (ignore)
-     int ignore;
+rl_list_funmap_names (count, ignore)
+     int count, ignore;
 {
   register int i;
   char **funmap_names;
@@ -1136,8 +1193,8 @@ rl_list_funmap_names (ignore)
 
 /* Return a NULL terminated array of strings which represent the key
    sequences that are used to invoke FUNCTION in MAP. */
-static char **
-invoking_keyseqs_in_map (function, map)
+char **
+rl_invoking_keyseqs_in_map (function, map)
      Function *function;
      Keymap map;
 {
@@ -1162,7 +1219,7 @@ invoking_keyseqs_in_map (function, map)
 	    {
 	      char *keyname = (char *)xmalloc (5);
 
-	      if (CTRL_P (key))
+	      if (CTRL_CHAR (key))
 		sprintf (keyname, "\\C-%c", to_lower (UNCTRL (key)));
 	      else if (key == RUBOUT)
 		sprintf (keyname, "\\C-?");
@@ -1195,11 +1252,7 @@ invoking_keyseqs_in_map (function, map)
 	       their target.  Add the key sequences found to RESULT. */
 	    if (map[key].function)
 	      seqs =
-#if defined (CRAY)
-		invoking_keyseqs_in_map (function, (Keymap)((int)map[key].function));
-#else
-		invoking_keyseqs_in_map (function, (Keymap)map[key].function);
-#endif
+	        rl_invoking_keyseqs_in_map (function, FUNCTION_TO_KEYMAP (map, key));
 
 	    if (seqs)
 	      {
@@ -1211,7 +1264,7 @@ invoking_keyseqs_in_map (function, map)
 
 		    if (key == ESC)
 		      sprintf (keyname, "\\e");
-		    else if (CTRL_P (key))
+		    else if (CTRL_CHAR (key))
 		      sprintf (keyname, "\\C-%c", to_lower (UNCTRL (key)));
 		    else if (key == RUBOUT)
 		      sprintf (keyname, "\\C-?");
@@ -1253,18 +1306,16 @@ char **
 rl_invoking_keyseqs (function)
      Function *function;
 {
-  return (invoking_keyseqs_in_map (function, _rl_keymap));
+  return (rl_invoking_keyseqs_in_map (function, _rl_keymap));
 }
 
 /* Print all of the current functions and their bindings to
    rl_outstream.  If an explicit argument is given, then print
    the output in such a way that it can be read back in. */
 int
-rl_dump_functions (count)
-     int count;
+rl_dump_functions (count, key)
+     int count, key;
 {
-  void rl_function_dumper ();
-
   rl_function_dumper (rl_explicit_arg);
   rl_on_new_line ();
   return (0);
@@ -1291,7 +1342,7 @@ rl_function_dumper (print_readably)
       char **invokers;
 
       function = rl_named_function (name);
-      invokers = invoking_keyseqs_in_map (function, _rl_keymap);
+      invokers = rl_invoking_keyseqs_in_map (function, _rl_keymap);
 
       if (print_readably)
 	{
@@ -1340,7 +1391,22 @@ rl_function_dumper (print_readably)
     }
 }
 
-
+/* Bind key sequence KEYSEQ to DEFAULT_FUNC if KEYSEQ is unbound. */
+void
+_rl_bind_if_unbound (keyseq, default_func)
+     char *keyseq;
+     Function *default_func;
+{
+  Function *func;
+
+  if (keyseq)
+    {
+      func = rl_function_of_keyseq (keyseq, _rl_keymap, (int *)NULL);
+      if (!func || func == rl_do_lowercase_version)
+	rl_set_key (keyseq, default_func, _rl_keymap);
+    }
+}
+
 /* **************************************************************** */
 /*								    */
 /*			String Utility Functions		    */
@@ -1363,6 +1429,7 @@ substring_member_of_array (string, array)
   return (0);
 }
 
+#if !defined (HAVE_STRCASECMP)
 /* Whoops, Unix doesn't have strnicmp. */
 
 /* Compare at most COUNT characters from string1 to string2.  Case
@@ -1380,7 +1447,8 @@ strnicmp (string1, string2, count)
       ch2 = *string2++;
       if (to_upper(ch1) == to_upper(ch2))
 	count--;
-      else break;
+      else
+        break;
     }
   return (count);
 }
@@ -1399,8 +1467,9 @@ stricmp (string1, string2)
       if (to_upper(ch1) != to_upper(ch2))
 	return (1);
     }
-  return (*string1 | *string2);
+  return (*string1 - *string2);
 }
+#endif /* !HAVE_STRCASECMP */
 
 /* Determine if s2 occurs in s1.  If so, return a pointer to the
    match in s1.  The compare is case insensitive. */
@@ -1412,7 +1481,7 @@ strindex (s1, s2)
   register int len = strlen (s1);
 
   for (i = 0; (len - i) >= l; i++)
-    if (strnicmp (&s1[i], s2, l) == 0)
+    if (strnicmp (s1 + i, s2, l) == 0)
       return (s1 + i);
   return ((char *)NULL);
 }
