@@ -20,6 +20,12 @@
    is generally kept in a file called COPYING or LICENSE.  If you do not
    have a copy of the license, write to the Free Software Foundation,
    675 Mass Ave, Cambridge, MA 02139, USA. */
+#define READLINE_LIBRARY
+
+#if defined (HAVE_CONFIG_H)
+#  include "config.h"
+#endif
+
 #include <sys/types.h>
 #include <signal.h>
 #include <errno.h>
@@ -141,6 +147,7 @@ control_meta_key (on)
     }
 }
 
+#if 0
 static void
 control_keypad (on)
      int on;
@@ -150,6 +157,7 @@ control_keypad (on)
   else if (!on && term_ke)
     tputs (term_ke, 1, outchar);
 }
+#endif
 
 /* **************************************************************** */
 /*								    */
@@ -196,6 +204,13 @@ get_tty_settings (tty, tiop)
      int tty;
      TIOTYPE *tiop;
 {
+#if !defined (SHELL) && defined (TIOCGWINSZ)
+  struct winsize w;
+
+  if (ioctl (tty, TIOCGWINSZ, &w) == 0)
+      (void) ioctl (tty, TIOCSWINSZ, &w);
+#endif
+
   tiop->flags = tiop->lflag = 0;
 
   ioctl (tty, TIOCGETP, &(tiop->sgttyb));
@@ -355,15 +370,33 @@ prepare_terminal_settings (meta_flag, otio, tiop)
 
 static TIOTYPE otio;
 
+#if defined (FLUSHO)
+#  define OUTPUT_BEING_FLUSHED(tp)  (tp->c_lflag & FLUSHO)
+#else
+#  define OUTPUT_BEING_FLUSHED(tp)  0
+#endif
+
 static int
 get_tty_settings (tty, tiop)
      int tty;
      TIOTYPE *tiop;
 {
-  while (GETATTR (tty, tiop) < 0)
+  int ioctl_ret;
+#if !defined (SHELL) && defined (TIOCGWINSZ)
+  struct winsize w;
+
+  if (ioctl (tty, TIOCGWINSZ, &w) == 0)
+      (void) ioctl (tty, TIOCSWINSZ, &w);
+#endif
+
+  /* Keep looping if output is being flushed after a ^O (or whatever
+     the flush character is). */
+  while ((ioctl_ret = GETATTR (tty, tiop)) < 0 || OUTPUT_BEING_FLUSHED (tiop))
     {
-      if (errno != EINTR)
+      if (ioctl_ret < 0 && errno != EINTR)
 	return -1;
+      if (OUTPUT_BEING_FLUSHED (tiop))
+        continue;
       errno = 0;
     }
   return 0;
@@ -439,6 +472,14 @@ prepare_terminal_settings (meta_flag, otio, tiop)
   tiop->c_cc[VMIN] = 1;
   tiop->c_cc[VTIME] = 0;
 
+#if defined (FLUSHO)
+  if (OUTPUT_BEING_FLUSHED (tiop))
+    {
+      tiop->c_lflag &= ~FLUSHO;
+      otio.c_lflag &= ~FLUSHO;
+    }
+#endif
+
   /* Turn off characters that we need on Posix systems with job control,
      just to be sure.  This includes ^Y and ^V.  This should not really
      be necessary.  */
@@ -488,7 +529,10 @@ rl_prep_terminal (meta_flag)
     }
 
   control_meta_key (1);
+#if 0
   control_keypad (1);
+#endif
+  fflush (rl_outstream);
   terminal_prepped = 1;
 
   release_sigint ();
@@ -508,14 +552,18 @@ rl_deprep_terminal ()
   /* Try to keep this function from being INTerrupted. */
   block_sigint ();
 
+  control_meta_key (0);
+#if 0
+  control_keypad (0);
+#endif
+  fflush (rl_outstream);
+
   if (set_tty_settings (tty, &otio) < 0)
     {
       release_sigint ();
       return;
     }
 
-  control_meta_key (0);
-  control_keypad (0);
   terminal_prepped = 0;
 
   release_sigint ();
@@ -601,79 +649,56 @@ rltty_set_default_bindings (kmap)
 
 #if defined (NEW_TTY_DRIVER)
 
+#define SET_SPECIAL(sc, func) \
+  do \
+    { \
+      int ic; \
+      ic = sc; \
+      if (ic != -1 && kmap[ic].type == ISFUNC) \
+	kmap[ic].function = func; \
+    } \
+  while (0)
+
   if (get_tty_settings (tty, &ttybuff) == 0)
     {
       if (ttybuff.flags & SGTTY_SET)
 	{
-	  int erase, kill;
-
-	  erase = ttybuff.sgttyb.sg_erase;
-	  kill  = ttybuff.sgttyb.sg_kill;
-
-	  if (erase != -1 && kmap[erase].type == ISFUNC)
-	    kmap[erase].function = rl_rubout;
-
-	  if (kill != -1 && kmap[kill].type == ISFUNC)
-	    kmap[kill].function = rl_unix_line_discard;
+	  SET_SPECIAL (ttybuff.sgttyb.sg_erase, rl_rubout);
+	  SET_SPECIAL (ttybuff.sgttyb.sg_kill, rl_unix_line_discard);
 	}
 
 #  if defined (TIOCGLTC)
-
       if (ttybuff.flags & LTCHARS_SET)
 	{
-	  int werase, nextc;
-
-	  werase = ttybuff.ltchars.t_werasc;
-	  nextc = ttybuff.ltchars.t_lnextc;
-
-	  if (werase != -1 && kmap[werase].type == ISFUNC)
-	    kmap[werase].function = rl_unix_word_rubout;
-
-	  if (nextc != -1 && kmap[nextc].type == ISFUNC)
-	    kmap[nextc].function = rl_quoted_insert;
+	  SET_SPECIAL (ttybuff.ltchars.t_werasc, rl_unix_word_rubout);
+	  SET_SPECIAL (ttybuff.ltchars.t_lnextc, rl_quoted_insert);
 	}
-    }
 #  endif /* TIOCGLTC */
+    }
 
 #else /* !NEW_TTY_DRIVER */
 
+#define SET_SPECIAL(sc, func) \
+  do \
+    { \
+      unsigned char uc; \
+      uc = ttybuff.c_cc[sc]; \
+      if (uc != (unsigned char)_POSIX_VDISABLE && kmap[uc].type == ISFUNC) \
+	kmap[uc].function = func; \
+    } \
+  while (0)
+
   if (get_tty_settings (tty, &ttybuff) == 0)
     {
-      unsigned char erase, kill;
-
-      erase = ttybuff.c_cc[VERASE];
-      kill = ttybuff.c_cc[VKILL];
-
-      if (erase != (unsigned char)_POSIX_VDISABLE &&
-	  kmap[erase].type == ISFUNC)
-	kmap[erase].function = rl_rubout;
-
-      if (kill != (unsigned char)_POSIX_VDISABLE &&
-	  kmap[kill].type == ISFUNC)
-	kmap[kill].function = rl_unix_line_discard;
+      SET_SPECIAL (VERASE, rl_rubout);
+      SET_SPECIAL (VKILL, rl_unix_line_discard);
 
 #  if defined (VLNEXT) && defined (TERMIOS_TTY_DRIVER)
-      {
-	unsigned char nextc;
-
-	nextc = ttybuff.c_cc[VLNEXT];
-
-	if (nextc != (unsigned char)_POSIX_VDISABLE &&
-	    kmap[nextc].type == ISFUNC)
-	  kmap[nextc].function = rl_quoted_insert;
-      }
+      SET_SPECIAL (VLNEXT, rl_quoted_insert);
 #  endif /* VLNEXT && TERMIOS_TTY_DRIVER */
 
 #  if defined (VWERASE) && defined (TERMIOS_TTY_DRIVER)
-      {
-	unsigned char werase;
-
-	werase = ttybuff.c_cc[VWERASE];
-
-	if (werase != (unsigned char)_POSIX_VDISABLE &&
-	    kmap[werase].type == ISFUNC)
-	  kmap[werase].function = rl_unix_word_rubout;
-      }
+      SET_SPECIAL (VWERASE, rl_unix_word_rubout);
 #  endif /* VWERASE && TERMIOS_TTY_DRIVER */
     }
 #endif /* !NEW_TTY_DRIVER */
