@@ -21,7 +21,7 @@ with Bash; see the file COPYING.  If not, write to the Free Software
 Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 #include <sys/types.h>
-#include <sys/stat.h>
+#include "posixstat.h"
 #include <stdio.h>
 
 char *progname;
